/*
 * Created on 26.06.2005
 */
package org.tmatesoft.svn.core.client;

import java.io.File;

import org.tmatesoft.svn.core.auth.ISVNAuthenticationManager;
import org.tmatesoft.svn.core.internal.wc2.compat.SvnCodec;
import org.tmatesoft.svn.core.javahl17.SVNClientImpl;
import org.tmatesoft.svn.core.wc.DefaultSVNCommitParameters;
import org.tmatesoft.svn.core.wc2.ISvnCommitParameters;
import org.tmatesoft.svn.core.wc2.SvnCommit;


public class SVNClientEx17 extends SVNClientImpl implements ISVNClientInterfaceEx {
    private boolean myIsCommitMissingFiles;
    private String myProxyPassword;
    private String myProxyUserName;
    private int myProxyPort;
    private String myProxyHost;
    private int mySSHPort;
    private String mySSHUserName;
    private String mySSHPassphrase;
    private String mySSHKeyPath;
    private String mySSLPassphrase;
    private String mySSLCertPath;
    private String mySSH2Password;
    private String mySSH2UserName;
    private int mySSH2Port;
    protected boolean myIsStoreSSL;
    protected ISvnCommitParameters commitParams;

    public SVNClientEx17() {
        super(null);
    }
    
    public void setSSLCertificateCacheEnabled(boolean enabled) {
        if (enabled != isSSLCertificateCacheEnabled()) {
            myIsStoreSSL = enabled;
            updateSvnOperationsFactory();
        }
    }

    public boolean isSSLCertificateCacheEnabled() {
        return myIsStoreSSL;
    }

    public boolean isCredentialsCacheEnabled() {
        return this.getOptions().isAuthStorageEnabled();
    }    

    public void setCredentialsCacheEnabled(boolean cacheCredentials) {
        if (cacheCredentials != isCredentialsCacheEnabled()) {
        	this.getOptions().setAuthStorageEnabled(cacheCredentials);            
        	updateSvnOperationsFactory();
        }
        return;
    }
    
    public void setCommitMissedFiles(boolean commitMissingFiles) {
        myIsCommitMissingFiles = commitMissingFiles;
        commitParams = SvnCodec.commitParameters(new DefaultSVNCommitParameters() {
            public Action onMissingFile(File file) {
                return myIsCommitMissingFiles ? DELETE : SKIP;
            }
            public Action onMissingDirectory(File file) {
                return myIsCommitMissingFiles ? DELETE : ERROR;
            }
        });
    }
    
    protected SvnCommit getSVNCommit() {
    	SvnCommit commit = super.getSVNCommit();
    	if (commitParams != null) {
    		commit.setCommitParameters(commitParams);
    	}
    	return commit;
    }

    public boolean isCommitMissingFile() {
        return myIsCommitMissingFiles;
    }

    public void setClientSSLCertificate(String certPath, String passphrase) {
        mySSLCertPath = certPath;
        mySSLPassphrase = passphrase;
        updateSvnOperationsFactory();
    }

    public void setSSHCredentials(String userName, String privateKeyPath, String passphrase, int port) {
        mySSHKeyPath = privateKeyPath;
        mySSHPassphrase = passphrase;
        mySSHUserName = userName;
        mySSHPort = port;
        updateSvnOperationsFactory();
    }

    public void setSSHCredentials(String userName, String password, int port) {
        mySSH2Password = password;
        mySSH2UserName = userName;
        mySSH2Port = port;
        updateSvnOperationsFactory();
    }

    public void setProxy(String host, int port, String userName, String password) {
        myProxyHost = host;
        myProxyPort = port;
        myProxyUserName = userName;
        myProxyPassword = password;
        updateSvnOperationsFactory();
    }

    protected ISVNAuthenticationManager createAuthenticationManager(File configDir, String userName, String password, boolean allowSave) {
        SVNClientAuthenticationManager authManager = new SVNClientAuthenticationManager(configDir, allowSave, myIsStoreSSL, userName, password);
        authManager.setProxy(myProxyHost, myProxyPort, myProxyUserName, myProxyPassword);
        authManager.setClientSSLCertificate(mySSLCertPath, mySSLPassphrase);
        authManager.setSSHCredentials(mySSHUserName, mySSHKeyPath, mySSHPassphrase, mySSHPort);
        authManager.setSSHCredentials(mySSH2UserName, mySSH2Password, mySSH2Port);
        return authManager;
    }
}
